@@ -27,10 +27,7 @@
 #include <src/udiskslogging.h>
 
 #include "udisksiscsitypes.h"
-#include "udisksiscsistate.h"
 #include "udisksiscsiutil.h"
-#include "udisksiscsidbusutil.h"
-#include "udisks-iscsi-generated.h"
 
 #ifndef HAVE_LIBISCSI_ERR
 /* XXX: We need to expose these in libiscsi.h.  If we can't make it appear in
@@ -104,24 +101,10 @@ enum {
 };
 #endif /* HAVE_LIBISCSI_ERR */
 
+
 const gchar *iscsi_nodes_fmt = "a(sisis)";
 const gchar *iscsi_node_fmt = "(sisis)";
 
-static struct libiscsi_context *
-iscsi_get_libiscsi_context (UDisksDaemon *daemon)
-{
-  UDisksISCSIState *state;
-  UDisksModuleManager *module_manager;
-
-  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), NULL);
-
-  module_manager = udisks_daemon_get_module_manager (daemon);
-  state = udisks_module_manager_get_module_state_pointer (module_manager,
-                                                          ISCSI_MODULE_NAME);
-
-  return udisks_iscsi_state_get_libiscsi_context (state);
-}
-
 static void
 iscsi_make_auth_info (struct libiscsi_auth_info *auth_info,
                       const gchar               *username,
@@ -174,7 +157,7 @@ iscsi_make_node (struct libiscsi_node *node,
 }
 
 static gint
-iscsi_perform_login_action (UDisksDaemon             *daemon,
+iscsi_perform_login_action (UDisksLinuxModuleISCSI     *module,
                             libiscsi_login_action       action,
                             struct libiscsi_node       *node,
                             struct libiscsi_auth_info  *auth_info,
@@ -183,16 +166,13 @@ iscsi_perform_login_action (UDisksDaemon             *daemon,
   struct libiscsi_context *ctx;
   gint err;
 
-  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), 1);
+  g_return_val_if_fail (UDISKS_IS_LINUX_MODULE_ISCSI (module), 1);
 
   /* Get a libiscsi context. */
-  ctx = iscsi_get_libiscsi_context (daemon);
+  ctx = udisks_linux_module_iscsi_get_libiscsi_context (module);
 
-  if (action == ACTION_LOGIN &&
-      auth_info && auth_info->method == libiscsi_auth_chap)
-    {
-      libiscsi_node_set_auth (ctx, node, auth_info);
-    }
+  if (action == ACTION_LOGIN && auth_info)
+    libiscsi_node_set_auth (ctx, node, auth_info);
 
   /* Login or Logout */
   err = action == ACTION_LOGIN ?
@@ -206,9 +186,10 @@ iscsi_perform_login_action (UDisksDaemon             *daemon,
 }
 
 static gint
-iscsi_node_set_parameters (struct libiscsi_context *ctx,
-                           struct libiscsi_node    *node,
-                           GVariant                *params)
+iscsi_node_set_parameters (struct libiscsi_context  *ctx,
+                           struct libiscsi_node     *node,
+                           GVariant                 *params,
+                           gchar                   **errorstr)
 {
   GVariantIter  iter;
   GVariant     *value;
@@ -227,9 +208,11 @@ iscsi_node_set_parameters (struct libiscsi_context *ctx,
 
       /* Update the node parameter value. */
       err = libiscsi_node_set_parameter (ctx, node, key, param_value);
+      if (errorstr && err != 0)
+        *errorstr = g_strdup (libiscsi_get_error_string (ctx));
 
       g_variant_unref (value);
-      g_free ((gpointer) key);
+      g_free (key);
     }
 
   return 0;
@@ -282,14 +265,14 @@ iscsi_params_pop_chap_data (GVariant      *params,
 }
 
 gint
-iscsi_login (UDisksDaemon  *daemon,
-             const gchar   *name,
-             const gint     tpgt,
-             const gchar   *address,
-             const gint     port,
-             const gchar   *iface,
-             GVariant      *params,
-             gchar        **errorstr)
+iscsi_login (UDisksLinuxModuleISCSI *module,
+             const gchar            *name,
+             const gint              tpgt,
+             const gchar            *address,
+             const gint              port,
+             const gchar            *iface,
+             GVariant               *params,
+             gchar                 **errorstr)
 {
   struct libiscsi_context *ctx;
   struct libiscsi_auth_info auth_info = {0,};
@@ -299,9 +282,9 @@ iscsi_login (UDisksDaemon  *daemon,
   const gchar *password = NULL;
   const gchar *reverse_username = NULL;
   const gchar *reverse_password = NULL;
-  gint err;
+  gint err = 0;
 
-  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), 1);
+  g_return_val_if_fail (UDISKS_IS_LINUX_MODULE_ISCSI (module), 1);
 
   /* Optional data for CHAP authentication. We pop these parameters from the
    * dictionary; it then contains only iSCSI node parameters. */
@@ -322,19 +305,20 @@ iscsi_login (UDisksDaemon  *daemon,
   iscsi_make_node (&node, name, tpgt, address, port, iface);
 
   /* Get iscsi context. */
-  ctx = iscsi_get_libiscsi_context (daemon);
+  ctx = udisks_linux_module_iscsi_get_libiscsi_context (module);
+
+  /* Update node parameters. */
+  if (params)
+    err = iscsi_node_set_parameters (ctx, &node, params_without_chap, errorstr);
 
   /* Login */
-  err = iscsi_perform_login_action (daemon,
-                                    ACTION_LOGIN,
-                                    &node,
-                                    &auth_info,
-                                    errorstr);
-
-  if (err == 0 && params)
+  if (err == 0)
     {
-      /* Update node parameters. */
-      err = iscsi_node_set_parameters (ctx, &node, params_without_chap);
+      err = iscsi_perform_login_action (module,
+                                        ACTION_LOGIN,
+                                        &node,
+                                        &auth_info,
+                                        errorstr);
     }
 
   g_variant_unref (params_without_chap);
@@ -343,52 +327,52 @@ iscsi_login (UDisksDaemon  *daemon,
 }
 
 gint
-iscsi_logout (UDisksDaemon  *daemon,
-              const gchar   *name,
-              const gint     tpgt,
-              const gchar   *address,
-              const gint     port,
-              const gchar   *iface,
-              GVariant      *params,
-              gchar        **errorstr)
+iscsi_logout (UDisksLinuxModuleISCSI *module,
+              const gchar            *name,
+              const gint              tpgt,
+              const gchar            *address,
+              const gint              port,
+              const gchar            *iface,
+              GVariant               *params,
+              gchar                 **errorstr)
 {
   struct libiscsi_context *ctx;
   struct libiscsi_node node = {0,};
-  gint err;
+  gint err = 0;
 
-  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), 1);
+  g_return_val_if_fail (UDISKS_IS_LINUX_MODULE_ISCSI (module), 1);
 
   /* Create iscsi node. */
   iscsi_make_node (&node, name, tpgt, address, port, iface);
 
   /* Get iscsi context. */
-  ctx = iscsi_get_libiscsi_context (daemon);
+  ctx = udisks_linux_module_iscsi_get_libiscsi_context (module);
+
+  /* Update node parameters. */
+  if (params)
+    err = iscsi_node_set_parameters (ctx, &node, params, errorstr);
 
   /* Logout */
-  err = iscsi_perform_login_action (daemon,
-                                    ACTION_LOGOUT,
-                                    &node,
-                                    NULL,
-                                    errorstr);
-
-  if (err == 0 && params)
+  if (err == 0)
     {
-      /* Update node parameters. */
-      err = iscsi_node_set_parameters (ctx, &node, params);
-
+      err = iscsi_perform_login_action (module,
+                                        ACTION_LOGOUT,
+                                        &node,
+                                        NULL,
+                                        errorstr);
     }
 
   return err;
 }
 
 gint
-iscsi_discover_send_targets (UDisksDaemon   *daemon,
-                             const gchar    *address,
-                             const guint16   port,
-                             GVariant       *params,
-                             GVariant      **nodes,
-                             gint           *nodes_cnt,
-                             gchar         **errorstr)
+iscsi_discover_send_targets (UDisksLinuxModuleISCSI *module,
+                             const gchar            *address,
+                             const guint16           port,
+                             GVariant               *params,
+                             GVariant              **nodes,
+                             gint                   *nodes_cnt,
+                             gchar                 **errorstr)
 {
   struct libiscsi_context *ctx;
   struct libiscsi_auth_info auth_info = {0,};
@@ -399,9 +383,9 @@ iscsi_discover_send_targets (UDisksDaemon   *daemon,
   const gchar *reverse_password = NULL;
   gint err;
 
-  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), 1);
+  g_return_val_if_fail (UDISKS_IS_LINUX_MODULE_ISCSI (module), 1);
 
-  ctx = iscsi_get_libiscsi_context (daemon);
+  ctx = udisks_linux_module_iscsi_get_libiscsi_context (module);
 
   /* Optional data for CHAP authentication. */
   iscsi_params_get_chap_data (params,
