@@ -6,6 +6,7 @@ import os
 import re
 import six
 import time
+import shutil
 import unittest
 
 
@@ -26,6 +27,13 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
     chap_iqn = 'iqn.2003-01.udisks.test:iscsi-test-chap'
     mutual_iqn = 'iqn.2003-01.udisks.test:iscsi-test-mutual'
 
+
+    # Define common D-Bus method call timeout that needs to be slightly longer
+    # than the corresponding timeout defined in libiscsi:
+    #   #define ISCSID_REQ_TIMEOUT 1000
+    # In reality the timeout is typically around 120 sec for the 'login' operation.
+    iscsi_timeout = 1000 + 5
+
     @classmethod
     def setUpClass(cls):
         udiskstestcase.UdisksTestCase.setUpClass()
@@ -40,9 +48,21 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
     def _set_initiator_name(self):
         manager = self.get_object('/Manager')
 
+        # make backup of INITIATOR_FILE and restore it at the end
+        try:
+            initiatorname_backup = self.read_file(INITIATOR_FILE)
+            self.addCleanup(self.write_file, INITIATOR_FILE, initiatorname_backup)
+        except FileNotFoundError as e:
+            # no existing file, simply remove it once finished
+            self.addCleanup(self.remove_file, INITIATOR_FILE, True)
+
         manager.SetInitiatorName(self.initiator, self.no_options,
                                  dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
 
+        # running iscsid needs to be restarted to reflect the change
+        self.run_command('systemctl try-reload-or-restart iscsid.service')
+        # ignore the return code in case of non-systemd distros
+
         init = manager.GetInitiatorName(self.no_options,
                                         dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
         self.assertEqual(init, self.initiator)
@@ -55,6 +75,10 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
         initiator = bytearray(data)
         return initiator.strip().split(b"InitiatorName=")[1]
 
+    def _clean_iscsid_node_dir(self):
+        for iqn in [self.noauth_iqn, self.chap_iqn, self.mutual_iqn]:
+            shutil.rmtree(os.path.join('/var/lib/iscsi/nodes/', iqn), ignore_errors=True)
+
     def test__manager_interface(self):
         '''Test for module D-Bus Manager interface presence'''
 
@@ -78,7 +102,9 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
     def test_login_noauth(self):
         manager = self.get_object('/Manager')
         nodes, _ = manager.DiscoverSendTargets(self.address, self.port, self.no_options,
-                                               dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                                               dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                                               timeout=self.iscsi_timeout)
+        self.addCleanup(self._clean_iscsid_node_dir)
 
         node = next((node for node in nodes if node[0] == self.noauth_iqn), None)
         self.assertIsNotNone(node)
@@ -90,7 +116,8 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
 
         self.addCleanup(self._force_lougout, self.noauth_iqn)
         manager.Login(iqn, tpg, host, port, iface, self.no_options,
-                      dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                      dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                      timeout=self.iscsi_timeout)
 
         devs = glob.glob('/dev/disk/by-path/*%s*' % iqn)
         self.assertEqual(len(devs), 1)
@@ -105,7 +132,8 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
         self.assertIn(self.str_to_ay(devs[0]), symlinks)
 
         manager.Logout(iqn, tpg, host, port, iface, self.no_options,
-                       dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                       dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                       timeout=self.iscsi_timeout)
 
         devs = glob.glob('/dev/disk/by-path/*%s*' % iqn)
         self.assertEqual(len(devs), 0)
@@ -120,7 +148,9 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
 
         manager = self.get_object('/Manager')
         nodes, _ = manager.DiscoverSendTargets(self.address, self.port, self.no_options,
-                                               dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                                               dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                                               timeout=self.iscsi_timeout)
+        self.addCleanup(self._clean_iscsid_node_dir)
 
         node = next((node for node in nodes if node[0] == self.chap_iqn), None)
         self.assertIsNotNone(node)
@@ -131,21 +161,30 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
         self.assertEqual(port, self.port)
 
         options = dbus.Dictionary(signature='sv')
+        options['node.session.auth.chap_algs'] = 'SHA3-256,SHA256,SHA1'  # disallow MD5
         options['username'] = self.initiator
 
+        msg = 'Login failed: initiator reported error \(24 - iSCSI login failed due to authorization failure\)'
+        # missing auth info
+        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+            manager.Login(iqn, tpg, host, port, iface, self.no_options,
+                          dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                          timeout=self.iscsi_timeout)
+
         # wrong password
-        msg = 'Login failed: initiator reported error'
         with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
             options['password'] = '12345'
             manager.Login(iqn, tpg, host, port, iface, options,
-                          dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                          dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                          timeout=self.iscsi_timeout)
 
         # right password
         options['password'] = self.password
 
         self.addCleanup(self._force_lougout, self.chap_iqn)
         manager.Login(iqn, tpg, host, port, iface, options,
-                      dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                      dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                      timeout=self.iscsi_timeout)
 
         devs = glob.glob('/dev/disk/by-path/*%s*' % iqn)
         self.assertEqual(len(devs), 1)
@@ -160,7 +199,8 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
         self.assertIn(self.str_to_ay(devs[0]), symlinks)
 
         manager.Logout(iqn, tpg, host, port, iface, self.no_options,
-                       dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                       dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                       timeout=self.iscsi_timeout)
 
         devs = glob.glob('/dev/disk/by-path/*%s*' % iqn)
         self.assertEqual(len(devs), 0)
@@ -175,7 +215,9 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
 
         manager = self.get_object('/Manager')
         nodes, _ = manager.DiscoverSendTargets(self.address, self.port, self.no_options,
-                                               dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                                               dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                                               timeout=self.iscsi_timeout)
+        self.addCleanup(self._clean_iscsid_node_dir)
 
         node = next((node for node in nodes if node[0] == self.mutual_iqn), None)
         self.assertIsNotNone(node)
@@ -186,6 +228,7 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
         self.assertEqual(port, self.port)
 
         options = dbus.Dictionary(signature='sv')
+        options['node.session.auth.chap_algs'] = 'SHA3-256,SHA256,SHA1'  # disallow MD5
         options['username'] = self.initiator
         options['password'] = self.password
         options['reverse-username'] = self.mutual_iqn
@@ -193,7 +236,8 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
 
         self.addCleanup(self._force_lougout, self.mutual_iqn)
         manager.Login(iqn, tpg, host, port, iface, options,
-                      dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                      dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                      timeout=self.iscsi_timeout)
 
         devs = glob.glob('/dev/disk/by-path/*%s*' % iqn)
         self.assertEqual(len(devs), 1)
@@ -208,7 +252,8 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
         self.assertIn(self.str_to_ay(devs[0]), symlinks)
 
         manager.Logout(iqn, tpg, host, port, iface, self.no_options,
-                       dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                       dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                       timeout=self.iscsi_timeout)
 
         devs = glob.glob('/dev/disk/by-path/*%s*' % iqn)
         self.assertEqual(len(devs), 0)
@@ -228,7 +273,9 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
             self.skipTest("ISCSI.Session objects not supported.")
 
         nodes, _ = manager.DiscoverSendTargets(self.address, self.port, self.no_options,
-                                               dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                                               dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                                               timeout=self.iscsi_timeout)
+        self.addCleanup(self._clean_iscsid_node_dir)
 
         node = next((node for node in nodes if node[0] == self.noauth_iqn), None)
         self.assertIsNotNone(node)
@@ -237,7 +284,8 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
 
         self.addCleanup(self._force_lougout, self.noauth_iqn)
         manager.Login(iqn, tpg, host, port, iface, self.no_options,
-                      dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator')
+                      dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                      timeout=self.iscsi_timeout)
 
         # /org/freedesktop/UDisks2/iscsi/sessionX should be created
         udisks = self.get_object('')
@@ -260,8 +308,69 @@ class UdisksISCSITest(udiskstestcase.UdisksTestCase):
 
         # logout using session
         session.Logout(self.no_options,
-                       dbus_interface=self.iface_prefix + '.ISCSI.Session')
+                       dbus_interface=self.iface_prefix + '.ISCSI.Session',
+                       timeout=self.iscsi_timeout)
 
         # make sure the session object is no longer on dbus
         objects = udisks.GetManagedObjects(dbus_interface='org.freedesktop.DBus.ObjectManager')
         self.assertNotIn(session_path, objects.keys())
+
+    def test_login_noauth_badauth(self):
+        """
+        Test auth info override
+        """
+        manager = self.get_object('/Manager')
+        nodes, _ = manager.DiscoverSendTargets(self.address, self.port, self.no_options,
+                                               dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                                               timeout=self.iscsi_timeout)
+        self.addCleanup(self._clean_iscsid_node_dir)
+
+        node = next((node for node in nodes if node[0] == self.noauth_iqn), None)
+        self.assertIsNotNone(node)
+
+        (iqn, tpg, host, port, iface) = node
+        self.assertEqual(iqn, self.noauth_iqn)
+        self.assertEqual(host, self.address)
+        self.assertEqual(port, self.port)
+
+        self.addCleanup(self._force_lougout, self.noauth_iqn)
+
+        # first attempt - wrong password
+        options = dbus.Dictionary(signature='sv')
+        options['node.session.auth.chap_algs'] = 'SHA3-256,SHA256,SHA1'  # disallow MD5
+        options['username'] = self.initiator
+        msg = r'Login failed: initiator reported error \((19 - encountered non-retryable iSCSI login failure|24 - iSCSI login failed due to authorization failure)\)'
+        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+            options['password'] = '12345'
+            manager.Login(iqn, tpg, host, port, iface, options,
+                          dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                          timeout=self.iscsi_timeout)
+
+        # second atttempt - no password
+        manager.Login(iqn, tpg, host, port, iface, self.no_options,
+                      dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                      timeout=self.iscsi_timeout)
+
+        devs = glob.glob('/dev/disk/by-path/*%s*' % iqn)
+        self.assertEqual(len(devs), 1)
+
+        # check if the block device have 'Symlinks' property updated
+        disk_name = os.path.realpath(devs[0]).split('/')[-1]
+        disk_obj = self.get_object('/block_devices/' + disk_name)
+        dbus_path = str(disk_obj.object_path)
+        self.assertIsNotNone(disk_obj)
+
+        symlinks = self.get_property_raw(disk_obj, '.Block', 'Symlinks')
+        self.assertIn(self.str_to_ay(devs[0]), symlinks)
+
+        manager.Logout(iqn, tpg, host, port, iface, self.no_options,
+                       dbus_interface=self.iface_prefix + '.Manager.ISCSI.Initiator',
+                       timeout=self.iscsi_timeout)
+
+        devs = glob.glob('/dev/disk/by-path/*%s*' % iqn)
+        self.assertEqual(len(devs), 0)
+
+        # make sure the disk is no longer on dbus
+        udisks = self.get_object('')
+        objects = udisks.GetManagedObjects(dbus_interface='org.freedesktop.DBus.ObjectManager')
+        self.assertNotIn(dbus_path, objects.keys())
